#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "vector.h"
#include "iotools.h"


#ifdef DEBUG_VECTOR
 unsigned long Acv; ///< alocation counter for vector class
 unsigned long Avmax; ///< peak of alocation memory of all vector instances
 unsigned long Ava; ///< actual allocated memory of all vector instances

 unsigned long Aciv; ///< alocation counter for ivector class
 unsigned long Aivmax; ///< peak of alocation memory of all ivector instances
 unsigned long Aiva; ///< actual allocated memory of all ivector instances
#endif




/**
   The destructor deallocates the memory occupied by the member array a
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
vector :: ~vector()
{
  #ifdef DEBUG_VECTOR
   if (a != NULL)
   {
     Acv--;
     Ava -= size;
   }
  #endif
  n = 0;
  a = NULL;
  size = 0L;
}



/**
   The constructor allocates memory form the heap to the member array a.

   @param n is the number of ivector elements
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
ivector :: ivector (long n)
{
  ivector::n = n;
  a = new long[n];
  if (a == NULL)
    print_err("cannot allocate memory for ivector v(%ld)", __FILE__, __LINE__, __func__, n);
  size = n;
  memset (a,0,n*sizeof(long));

  #ifdef DEBUG_VECTOR
   Aciv++;
   Aiva += n;
   if (Aiva > Aivmax)
     Aivmax = Aiva;
  #endif
}



/**
   The copy constructor creates copy of the object given by the constant reference parameter.
   
   @param v is reference to the object of the ivector which should be copied
   
   created  16.10.2003 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
ivector :: ivector (const ivector &v)
{
  long i;
  if (v.n > size)
  {
    n = v.n;
    a = new long[v.n];
    if (a == NULL)
      print_err("cannot allocate memory for ivector v(%ld)", __FILE__, __LINE__, __func__, n);
    size = v.n;
    #ifdef DEBUG_VECTOR
     print_warning("copy constructor of ivector is called.\n"
                   "Please check your code and make sure you want use copy constructor.", __FILE__, __LINE__, __func__);
     Aciv++;
     Aiva += n;
     if (Aiva > Aivmax)
       Aivmax = Aiva;
    #endif
  }
  for (i = 0; i < n; i++)
    a[i] = v.a[i];
}

/**
 The destructor deallocates the memory occupied by the member array a

 created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
 modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
ivector :: ~ivector()
{
  #ifdef DEBUG_VECTOR
   if (a != NULL)
   {
     Aciv--;
     Aiva -= size;
   }
  #endif
  n = 0;
  a = NULL;
  size = 0L;
}


#ifdef DEBUG_VECTOR
 long give_acv()
 {
   return Acv;
 }
 long give_avmax()
 {
   return Avmax;
 }
#endif



/**
   The function allocates memory from heap to the ivector's member array a.
   
   @param n   is the number of the ivector elements
   @param vec is the structure for allocated ivector
   
   @retval 0 - on success
   @retval 1 - if fails allocating memory
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long allocv(long n, ivector &vec)
{
  vec.n = n;
  vec.a = new long[n];
  if (vec.a == NULL)
  {
    print_err("cannot allocate memory for ivector v(%ld)", __FILE__, __LINE__, __func__, n);
    return (1);
  }
  vec.size = n;
  memset (vec.a,0,n*sizeof(*vec.a));
  #ifdef DEBUG_VECTOR
   Aciv++;
   Aiva += n;
   if (Aiva > Aivmax)
     Aivmax = Aiva;
  #endif
  return (0);
}



/**
   The function reallocates memory for the ivector's member array a.
   If the new ivector size is greater than previously allocated one, 
   array a is deleted and new memory is allocted from the heap. 
   If the new ivector size is less or equal, then the memory 
   allocated for the array a is left and the number of %ivector components 
   is changed. Components of the array a are set to zero.
   
   
   @param n   is the new number of the ivector components
   @param vec is the structure for reallocated ivector
   
   @retval 0 - on success
   @retval 1 - if fails allocating memory
   
   created  12.2012 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long reallocv(long n, ivector &vec)
{
  if (n > vec.n)
  {
    vec.a = new long[n];
    if (vec.a == NULL)
    {
      print_err("cannot reallocate memory for ivector v(%ld)", __FILE__, __LINE__, __func__, n);
      return (1);
    }
    vec.size = n;
    #ifdef DEBUG_VECTOR
     Aciv++;
     Aiva += n;
     if (Aiva > Aivmax)
       Aivmax = Aiva;
    #endif
  }
  vec.n = n;
  memset (vec.a,0,n*sizeof(*vec.a));
  return (0);
}



/**
   The function copies vector given by src to dest.
   
   @param src  is the structure of source vector to copy
   @param dest is the structure of destination vector to which will be copied contents of src
   
   @b Requests :
   dest has to be setuped dimensions and allocated memory array for elements
   which is enough large to hold all contents of the vector src.

   @retval 0 : on succes
   @retval 1 : in case incompatibility sizes of the src and dest vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long copyv(const vector &src, vector &dest)
{
  if (src.n != dest.n)
  {
    print_err("cannot copy vectors - incompatible dimensions of vectors\n"
              "src(%ld) X dest(%ld)", __FILE__, __LINE__, __func__, src.n, dest.n);
    return (1);
  }
  memcpy(dest.a, src.a, sizeof(double)*src.n);
  return (0);
}



/**
   The function copies %vector given by src to dest.
   
   @param src  is the structure of source %ivector to copy
   @param dest is the structure of destination %ivector to which will be copied contents of src
   
   @b Requests :
   dest has to be setuped dimensions and allocated memory array for elements
   which is enough large to hold all contents of the %ivector src.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of the src and dest vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long copyv(const ivector &src, ivector &dest)
{
  if (src.n != dest.n)
  {
    print_err("cannot copy ivector - incompatible dimensions of ivectors\n"
              "src(%ld) X dest(%ld)", __FILE__, __LINE__, __func__, src.n, dest.n);
    return (1);
  }
  memcpy(dest.a, src.a, sizeof(long)*src.n);
  return (0);
}



/**
   The function copies long array given by src to dest.
   
   @param src  is the double array to copy
   @param dest is the double array to which will be copied contents of src
   
   @retval always zero
   
   created  3.11.2001 by JK
*/
long copyv(const long *src,long *dest,long n)
{
  memcpy(dest, src, sizeof(long)*n);
  return (0);
}



/**
   The function copies double array given by src to dest.
   
   @param src  is the double array to copy
   @param dest is the double array to which will be copied contents of src
   
   @retval always zero
   
   created  3.11.2001 by JK
*/
long copyv (const double *src, double *dest, long n)
{
  memcpy(dest, src, sizeof(double)*n);
  return (0);
}



/**
   The function copies array given by src to vector dest.
   
   @param src  is source array to copy
   @param dest is the structure of destination vector which will be copied contents of src to
   
   @b Requests :
   dest has to be setuped dimensions and allocated memory array for required elements.
   Number of copied elements is given by the dest dimension

   @retval 0 : on succes
   
   created  4.10.2007 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
 */
long copyv(const double *src, vector &dest)
{
  memcpy(dest.a, src, sizeof(double)*dest.n);
  return (0);
}



/**
   The function copies vector given by src to array dest.
   
   @param src  is the structure of source vector to copy
   @param dest is the destination array which will be copied contents of src to
   
   @b Requests :
   dest has to have allocated memory which is enough large to hold all contents 
   of the vector src. Number of copied elements is given by the src dimension

   @retval 0 : on succes
   
   created  4.10.2007 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
 */
long copyv(const vector &src, double *dest)
{
  memcpy(dest, src.a, sizeof(double)*src.n);
  return (0);
}



/**
   The function copies vector given by src multiplied by c to dest.
   
   @param src  - structure with source %vector to copy
   @param dest - structure with destination %vector to which will be copied contents of the src
   @param c    - scalar multiplier
   
   @b Requests :
   dest has to have dimensions and allocated memory array for %vector components
   which is large enough to hold all contents of the %vector src.

   @retval 0 : on succes
   @retval 1 : in case incompatibility sizes of the src and dest vectors
   
   created  08.2011 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long copymultv(const vector &src, vector &dest, const double c)
{
  if (src.n != dest.n)
  {
    print_err("cannot copy vectors - incompatible dimensions of vectors\n"
              "src(%ld) X dest(%ld)", __FILE__, __LINE__, __func__, src.n, dest.n);
    return (1);
  }
  long i;  
  for (i=0; i<src.n; i++)
    dest[i] = c*src[i];
  return (0);
}



/**
   The function copies ivector given by src multiplied by scalar c to dest.
   
   @param src  - structure with the source ivector to copy
   @param dest - structure with the destination %ivector to which will be copied the contents of the src
   @param c    - scalar multiplier
   
   @b Requests :
   dest has to have dimensions and allocated memory array for %vector components
   which is large enough to hold all contents of the %ivector src.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of the src and dest vectors
   
   created  08.2011 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long copymultv(const ivector &src, ivector &dest, const long c)
{
  if (src.n != dest.n)
  {
    print_err("cannot copy ivector - incompatible dimensions of ivectors\n" 
              "src(%ld) X dest(%ld)", __FILE__, __LINE__, __func__, src.n, dest.n);
    return (1);
  }
  long i;  
  for (i=0; i<src.n; i++)
    dest[i] = c*src[i];
  return (0);
}



/**
   The function copies double array given by the src multiplied by scalar c to the dest.
   
   @param src  - double array to copy
   @param dest - double array to which will be copied contents of src
   @param c    - scalar multiplier
   @param n    - length of arrays src and dest
   
   @retval always zero
   
   created  08.2011 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long copymultv(const double *src, double *dest, const double c, long n)
{
  long i;  
  for (i=0; i<n; i++)
    dest[i] = c*src[i];
  return (0);
}



/**
   The function fills vect's member array a with value c
   
   @param c is value, which will be used for filling memory
   @param vec is the structure %vector for the deallocated %vector
   
   @return always zero
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long fillv(double c, vector &vec)
{
  for (long i = 0; i < vec.n; vec(i) = c, i++);
  return (0);
}



/**
   The function fills vect's member array a with value c
   
   @param c is value, which will be used for filling memory
   @param vec is the structure %ivector which should be filled
   
   @retval always zero
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long fillv(long c, ivector &vec)
{
  for (long i = 0; i < vec.n; vec(i) = c, i++);
  return (0);
}



/**
   The function fills vect's member array a with value c
   
   @param c is value, which will be used for filling memory
   @param vec is the structure %ivector which should be filled
   
   @retval always zero
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
void fillv(long c, long *vec,long n)
{
  long i;
  
  for (i=0;i<n;i++)
    vec[i]=c;
}



/**
   The function fills vect's member array a with value c
   
   @param c is value, which will be used for filling memory
   @param vec is the structure %ivector which should be filled
   
   @retval always zero
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
void fillv(double c, double *vec,long n)
{
  long i;
  
  for (i=0;i<n;i++)
    vec[i]=c;
}



/**
   The function fills double array a with zeros
   
   @param a is array which should be filled
   
   @retval always zero
*/
long nullv (double *a,long n)
{
  memset (a,0,n*sizeof(double));
  return (0);
}



long nullv (ivector &a)
{
  memset (a.a,0,a.n*sizeof(long));
  return (0);
}



/**
   The function deallocates memory occupied by %vector vec
   
   @param vec is the structure for the deallocated %vector
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long destrv(vector &vec)
{
  #ifdef DEBUG_VECTOR
   if (vec.a != NULL)
   {
     Acv--;
     Ava -= vec.n;
   }
  #endif
  vec.n = 0;
  vec.a = NULL;
  vec.size = 0L;
  return (0);
}



/**
   The function deallocates memory occupied by %ivector vec
   
   @param vec is the structure for the deallocated %ivector
   
   @retval 0
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long destrv(ivector &vec)
{
  #ifdef DEBUG_VECTOR
   if (vec.a != NULL)
   {
     Aciv--;
     Aiva -= vec.n;
   }     
  #endif
  vec.n = 0;
  vec.a = NULL;
  vec.size = 0L;
  return (0);
}



/**
   The function adds vector given by a to %vector given by b, the result is stored in c
   
   @param a is the structure of the first added vector
   @param b is the structure of the second added vector
   @param c is the structure of the result vector
   
   @b Requests :
   a, b and c have to be same dimensions, c has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 : on succes
   @retval 1 : in case incompatibility sizes of a, b and c vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long addv(const vector &a, const vector &b, vector &c)
{
  if ((a.n != b.n) || (b.n != c.n))
  {
    print_err("cannot add vectors due to their incompatible dimensions\n"
              "a(%ld) X b(%ld) X c(%ld)", __FILE__, __LINE__, __func__, a.n, b.n, c.n);
    return (1);
  }
  for (long i = 0; i < a.n; i++)
    c.a[i] = a.a[i] + b.a[i];

  return(0);
}



/**
   The function adds vector given by a to %vector given by b, the result is stored in c.
   
   @param a is the structure of the first added vector
   @param b is the structure of the second added vector
   @param c is the structure of the result vector
   
   @b Requests :
   a, b and c must be the same dimensions, c must have allocated memory array for elements
   which is large enough to hold contents of the result.
   
   @return The function returns the result in the parameter c.

   created  08.2011 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
void addmultv (const vector &a, const vector &b, double bc, vector &c)
{
  if ((a.n != b.n) || (b.n != c.n))
  {
    print_err("cannot add vectors due to their incompatible dimensions\n"
              "a(%ld) X b(%ld) X c(%ld)", __FILE__, __LINE__, __func__, a.n, b.n, c.n);
    return;
  }
  for (long i=0;i<a.n;i++)
    c[i] = a[i]+b[i]*bc;
}



/**
   The function adds vector given by a to %vector given by b which is multiplied by bc, the result is stored in a.
   
   @param a is the structure of the first added vector
   @param b is the structure of the second added vector
   
   @b Requests :
   a and b must have the same dimensions, c must have allocated memory array for elements
   which is large enough to hold contents of the result.
   
   @return The function returns the result in the parameter a.

   created  08.2011 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
void addmultv (vector &a, const vector &b, double bc)
{
  if (a.n != b.n)
  {
    print_err("cannot add vectors due to their incompatible dimensions\n"
              "a(%ld) X b(%ld)", __FILE__, __LINE__, __func__, a.n, b.n);
    return;
  }
  for (long i=0;i<a.n;i++)
    a[i] += b[i]*bc;
}



/**
   The function adds ivector given by a to ivector given by b, the result is stored in c
   
   @param a is the structure of the first added ivector
   @param b is the structure of the second added ivector
   @param c is the structure of the result ivector
   
   @b Requests :
   a, b and c should have same dimensions, c has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of a, b and c vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long addv(const ivector &a, const ivector &b, ivector &c)
{
  if ((a.n != b.n) || (b.n != c.n))
  {
    print_err("cannot add ivectors due to their incompatible dimensions\n"
              "a(%ld) X b(%ld) X c(%ld)", __FILE__, __LINE__, __func__, a.n, b.n, c.n);
    return (1);
  }
  for (long i = 0; i < a.n; i++)
    c.a[i] = a.a[i] + b.a[i];

  return(0);
}



/**
   The function adds double array b to double array given by a, the result is stored in a
   
   @param a is result array
   @param b is added array
   @param n is the number of vector components
   
   @retval always zero
   
   created  by JK
*/
void addv (double *a,double *b,long n)
{
  for (long i=0;i<n;i++)
    a[i]+=b[i];
}



/**
   The function adds two double arrays a and b, the result is stored in c
   
   @param a is added array
   @param b is added array
   @param c is result array
   @param n is the number of vector components
   
   created  by JK
*/
void addv (double *a,double *b,double *c,long n)
{
  for (long i=0;i<n;i++)
    c[i] = a[i]+b[i];
}



/**
   The function adds double array b multiplied by scalar c to double array given by a, the result is stored in a
   
   @param a is result array
   @param b is added array
   @param bc is scalar
   @param n is the number of vector components
   
   @return The result is stored in the array a.
   
   created  by JK
*/
void addmultv (double *a,double *b,double bc,long n)
{
  for (long i=0;i<n;i++)
    a[i]+=b[i]*bc;
}



/**
   The function adds double array b multiplied by scalar bc to double array given by a, the result is stored in c
   
   @param a is result array
   @param b is added array
   @param bc is scalar
   @param c is scalar
   @param n is the number of vector components
   
   @return The result is stored in the array c.

   created  by TKo
*/
void addmultv (double *a,double *b,double bc, double* c,long n)
{
  for (long i=0;i<n;i++)
    c[i] = a[i]+b[i]*bc;
}



/**
   The function subtracts vector given by b from %vector given by a, the result is stored in the c
   
   @param a is the structure of the %vector, from which is subtracted
   @param b is the structure of subtracted %vector
   @param c is the structure of the result %vector
   
   @b Requests :
   a, b and c have to be same dimensions, c has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of a, b and c vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long subv(const vector &a, const vector &b, vector &c)
{
  if ((a.n != b.n) || (b.n != c.n))
  {
    print_err("cannot subtract vectors due to their incompatible dimensions\n"
              "a(%ld) X b(%ld) X c(%ld)", __FILE__, __LINE__, __func__, a.n, b.n, c.n);
    return (1);
  }
  for (long i = 0; i < c.n; i++)
    c.a[i] = a.a[i] - b.a[i];

  return(0);
}



/**
   The function subtracts vector given by b from %vector given by a, the result is stored in the a
   
   @param a is the structure of the %vector, from which is subtracted
   @param b is the structure of subtracted %vector
   
   @b Requests :
   a and b have to have same dimensions
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of a and b vectors
   
   created  08.2011 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long subv(const vector &a, const vector &b)
{
  if (a.n != b.n)
  {
    print_err("cannot subtract vectors due to their incompatible dimensions\n"
              "a(%ld) X b(%ld)", __FILE__, __LINE__, __func__, a.n, b.n);
    return (1);
  }
  for (long i = 0; i < a.n; i++)
    a.a[i] -= b.a[i];

  return(0);
}



/**
   The function subtracts %ivector given by b from %ivector given by a, the result is stored in the c
   
   @param a is the structure of the %ivector, from which is subtracted
   @param b is the structure of subtracted %ivector
   @param c is the structure of the result %ivector
   
   @b Requests :
   a, b and c should have same dimensions, c has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of a, b and c ivectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long subv(const ivector &a, const ivector &b, ivector &c)
{
  if ((a.n != b.n) || (b.n != c.n))
  {
    print_err("cannot subtract vectors due to their incompatible dimensions\n"
              "a(%ld) X b(%ld) X c(%ld)", __FILE__, __LINE__, __func__, a.n, b.n, c.n);
    return (1);
  }
  for (long i = 0; i < c.n; i++)
    c.a[i] = a.a[i] - b.a[i];
      
  return(0);
}



/**
   The function subtracts double array given by b from double array given by a, the result is stored in the a
   
   @param a is the array, from which is subtracted
   @param b is the subtracted array
   @param n is number of members of arrays
   
   @retval always zero
   
   created by JK
*/
void subv (double *a,double *b,long n)
{
  for (long i=0;i<n;i++)
    a[i]-=b[i];
}



/**
  The function subtracts double array given by b from double array by a, the result is stored 
  in the array c.

  @param a - double array of minuend
  @param b - double array of subtrahent
  @param c - double array of resulting difference
  @param n - length of arrays a, b and c

  @return The result is returned in the parameter c.

  Created by JK
*/
void subv (double *a,double *b,double *c,long n)
{
  long i;
  
  for (i=0;i<n;i++){
    c[i] = a[i]-b[i];
  }
}



/**
   The function performs vector product, multiplies %vector given by a
   from right with %vector given by b, the result is stored in c
   
   @param a is the structure of the %vector, from which is multiplied
   @param b is the structure of the multiplicating %vector
   @param c is the structure of the result %vector
   
   @b Requests :
   a, b and c have to be same dimension equal 3
   c has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of a, b and c vectors
   @retval 2 in case a and b vectors have dimension inequal 3
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long crprd(const vector &a, const vector &b, vector &c)
{
  if ((a.n != b.n) || (b.n != c.n))
  {
    print_err("cannot perform cross product of vectors due to their incompatible dimensions\n"
              "a(%ld) X b(%ld) X c(%ld)", __FILE__, __LINE__, __func__, a.n, b.n, c.n);
    return (1);
  }
  if ((a.n != 3) || (b.n != 3))
  {
    print_err("cannot perform cross product of vectors due to wrong dimensions (!= 3)\n"
              "a(%ld) X b(%ld) X c(%ld)", __FILE__, __LINE__, __func__, a.n, b.n, c.n);
    return (2);
  }
  c.a[0] = a.a[1] * b.a[2] - a.a[2] * b.a[1];
  c.a[1] = a.a[2] * b.a[0] - a.a[0] * b.a[2];
  c.a[2] = a.a[0] * b.a[1] - a.a[1] * b.a[0];
  
  return (0);
}



/**
   The function performs vector product, multiplies %ivector given by a
   from right with %ivector given by b, the result is stored in c
   
   @param a is the structure of the %ivector, from which is multiplied
   @param b is the structure of the multiplicating %ivector
   @param c is the structure of the result %ivector
   
   @b Requests :
   a, b and c should have same dimension equal 3
   c has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of a, b and c vectors
   @retval 2 in case a and b vectors have dimension inequal 3
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long crprd(const ivector &a, const ivector &b, ivector &c)
{
  if ((a.n != b.n) || (b.n != c.n))
  {
    print_err("cannot perform cross product of ivectors due to their incompatible dimensions\n"
              "a(%ld) X b(%ld) X c(%ld)", __FILE__, __LINE__, __func__, a.n, b.n, c.n);
    return (1);
  }
  if ((a.n != 3) || (b.n != 3))
  {
    print_err("cannot perform cross product of ivectors due to wrong dimensions (!= 3)\n"
              "a(%ld) X b(%ld) X c(%ld)", __FILE__, __LINE__, __func__, a.n, b.n, c.n);
    return (2);
  }
  c.a[0] = a.a[1] * b.a[2] - a.a[2] * b.a[1];
  c.a[1] = a.a[2] * b.a[0] - a.a[0] * b.a[2];
  c.a[2] = a.a[0] * b.a[1] - a.a[1] * b.a[0];
  
  return (0);
}



/**
   The function performs scalar product of the %vector given by a and %vector given by b,
   the result is stored in the c.

  @param a is the structure of the scalar multiplied %vector
  @param b is the structure of the scalar multiplied %vector
  @param scalar is the real number type of double by which the result is passed

  @b Requests :
  a and b have to be same dimension
  
  @retval 0 : on succes
  @retval 1 : in case incompatibility sizes of a and b
  
  created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long scprd(const vector &a, const vector &b, double &scalar)
{
  if (a.n != b.n)
  {
    print_err("cannot perform scalar product of vectors due to their incompatible dimensions\n"
              "a(%ld) X b(%ld)", __FILE__, __LINE__, __func__, a.n, b.n);
    return (1);
  }
  scalar = 0.0;
  for (long i=0; i < a.n; i++)
    scalar += a.a[i] * b.a[i];
  return (0);
}



/**
  The function computes the scalar product of the %vector given by a and %vector given by b.

  @param a - onedimensional array of the first multiplied %vector
  @param b - onedimensional array of the second multiplied %vector
  @param n - number of components in arrays a and b.

  @b Requests :
  a and b must have the same dimension
  
  @return The function returns the scalar product of vectors a and b.

  created  12.2011 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
double scprd(const double *a, const double *b, long n)
{
  double scalar = 0.0;
  for (long i=0; i < n; i++)
    scalar += a[i] * b[i];
  return (scalar);
}



/**
   The function performs scalar product of the %ivector given by a and %ivector given by b,
   the result is stored in the c
   
   @param a is the structure of the scalar multiplied %ivector
   @param b is the structure of the scalar multiplied %ivector
   @param scalar is the integer number type of long by which the result is passed
   
   @b Requests :
   a and b should have same dimension
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of a and b
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long scprd(const ivector &a, const ivector &b, long &scalar)
{
  if (a.n != b.n)
  {
    print_err("cannot perform scalar product of ivectors due to their incompatible dimensions\n"
              "a(%ld) X b(%ld)", __FILE__, __LINE__, __func__, a.n, b.n);
    return (1);
  }
  scalar = 0;
  for (long i=0; i < a.n; i++)
    scalar += a.a[i] * b.a[i];
  return (0);
}



/**
   The function performs scalar product of the double array given by a and double array given by b,
   the result is stored in the c
   
   @param a is the first array
   @param b is the second array
   @param n is number of members of arrays
   
   created  by JK
*/
double ss (double *a,double *b,long n)
{
  double s = 0.0;
  for (long i=0;i<n;i++)
    s+=a[i]*b[i];
  
  return s;
}



/**
   The function multiplies %vector given by u by real constant c,
   the result is stored in the double %vector v
   
   @param c is the real number type double
   @param u is the structure of the %vector, which is multiplied
   @param v is the structure of the result %vector
   
   @b Requests :
   u and v have to be same dimensions, u has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 : on succes
   @retval 1 : in case incompatibility sizes of u and v vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cmulv(double c, const vector &u, vector &v)
{
  if (u.n != v.n)
  {
    print_err("cannot multiply vector by constant due to their incompatible dimensions\n"
              "u(%ld) X v(%ld)", __FILE__, __LINE__, __func__, u.n, v.n);
    return (1);
  }
  for (long i=0; i < u.n; i++)
    v.a[i] = c * u.a[i];

  return (0);
}



/**
   The function multiplies %ivector given by u by real constant c,
   the result is stored in the double %ivector v
   
   @param c is the real number type double
   @param u is the structure of the %ivector, which is multiplied
   @param v is the structure of the result %vector
   
   @b Requests :
   u and v should have same dimensions, u has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of u and v vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cmulv(double c, const ivector &u, vector &v)
{
  if (u.n != v.n)
  {
    print_err("cannot multiply ivector by constant due to their incompatible dimensions\n"
              "u(%ld) X v(%ld)", __FILE__, __LINE__, __func__, u.n, v.n);
    return (1);
  }
  for (long i=0; i < u.n; i++)
    v.a[i] = c * u.a[i];
  
  return (0);
}



/**
   The function multiplies %ivector given by u by integer constant c,
   the result is stored in the %vector v
   
   @param c is the integer number type long
   @param u is the structure of the %ivector, which is multiplied
   @param v is the structure of the result %ivector
   
   @b Requests :
   u and v should have same dimensions, u has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of u and v %vector
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cmulv(long c, const ivector &u, ivector &v)
{
  if (u.n != v.n)
  {
    print_err("cannot multiply ivector by constant due to their incompatible dimensions\n"
              "u(%ld) X v(%ld)", __FILE__, __LINE__, __func__, u.n, v.n);
    return (1);
  }
  for (long i=0; i < u.n; i++)
    v.a[i] = c * u.a[i];
  
  return (0);
}



/**
   The function multiplies %vector given by a with constant c,
   the result is stored in a
   
   @param c is the real number
   @param a is the structure of the %vector, which is multiplied
   
   @return always zero
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cmulv(double c, vector &a)
{
  for (long i=0; i < a.n; i++)
    a.a[i] *= c;
  
  return (0);
}



/**
   The function multiplies %ivector given by a with integer constant c,
   the result is stored in a
   
   @param c is the integer number
   @param a is the structure of the %ivector, which is multiplied
   
   @retval 0 always
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cmulv(long c, ivector &a)
{
  for (long i=0; i < a.n; i++)
    a.a[i] *= c;
  
  return (0);
}



/**
   The function multiplies double array given by a with constant c,
   the result is stored in a
   
   @param c is the real number
   @param a is the array, which is multiplied
   @param n is number of members of arrays
   
   @return always zero
   
   created by JK
*/
void cmulv(double c,double *a,long n)
{
  for (long i=0; i < n; i++)
    a[i] *= c;
  
}



/**
   The function multiplies double array given by a with constant c,
   the result is stored in u.
   
   @param c is the real number
   @param a is the array, which is multiplied
   @param u is the array, where the result is stored
   @param n is number of members of arrays
   
   @return always zero
   
   created by JK
*/
void cmulv(double c,double *a, double *u, long n)
{
  for (long i=0; i < n; i++)
    u[i] = c*a[i];
}



/**
   The function computes norm of %vector given by a.
   
   @param a - structure of %vector components 
   
   @return The function returns computed norm of the %vector.
   
   created  02.2010 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
double normv(const vector &a)
{
  double s = 0.0;
  for (long i=0; i < a.n; i++)
    s += a.a[i] * a.a[i];
  return sqrt(s);
}



/*
  The function computes norm of the %vector a.

  @param a - array with vector values
  @param n - number of %vector components (i.e. size of the array a)
  
  @return The function returns computed norm of the %vector.

  created  02.2010 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
double normv(const double *a,long n)
{
  long i;
  double s;
  
  s=0.0;
  for (i=0;i<n;i++){
    s+=a[i]*a[i];
  }
  return sqrt(s);
}



/**
   The function computes norm of the %ivector given by a,
   the result is stored in the parameter norm
   
   @param a is the structure of the ivector, which size is requested
   
   @return Returns computed size of %vector norm.
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
double normv(const ivector &a)
{
  double norm = 0.0;
  for (long i=0; i < a.n; i++)
    norm += a.a[i] * a.a[i];
  norm = sqrt(norm);
  
  return(norm);
}



/**
   The function computes cosine of the angle between %vector given by a and b
   the result is stored in the cos
   
   @param a is the structure of the first %vector
   @param b is the structure of the second %vector
   @param cos is real number, type double, where result is stored
   
   @b Requests :
   vectors a nd b should have the same dimension
   
   @retval 0 on succes
   @retval 1 in case
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cosav(const vector &a, const vector &b, double &cos)
{
  if (a.n != b.n)
  {
    print_err("cannot determine cos angle of vectors due to their incompatible dimensions\n"
              "a(%ld) X b(%ld)", __FILE__, __LINE__, __func__, a.n, b.n);
    return (1);
  }
  
  double scalar, norm_a, norm_b;
  scprd(a, b, scalar);
  norm_a = normv(a);
  norm_b = normv(b);
  cos = scalar/(norm_a * norm_b);
  
  return (0);
}



/**
   The function computes cosine of the angle between %ivector given by a and b
   the result is stored in the cos
   
   @param a is the structure of the first %ivector
   @param b is the structure of the second %ivector
   @param cos is real number, type double, where result is stored
   
   @b Requests :
   vectors a nd b should have the same dimension

   @retval 0 on succes
   @retval 1 in case
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cosav(const ivector &a, const ivector &b, double &cos)
{
  if (a.n != b.n)
  {
    print_err("cannot determine cos angle of ivectors due to their incompatible dimensions\n"
              "a(%ld) X b(%ld)", __FILE__, __LINE__, __func__, a.n, b.n);
    return (1);
  }
  
  long scalar;
  double norm_a, norm_b;
  scprd(a, b, scalar);
  norm_a = normv(a);
  norm_b = normv(b);
  cos = scalar/(norm_a * norm_b);
  
  return (0);
}



/**
   The function changes components of %vector a to their absolute value
   
   @param a is the structure of the first %ivector
   
   created  7.2008 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
void absv(vector &a)
{
  long i;
  for (i=0; i<a.n; i++)
    a[i] = fabs(a[i]);
}



/**
   The function changes components of %vector a to their absolute value
   
   @param a is the structure of the first %ivector
   
   created  7.2008 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
void absv(ivector &a)
{
  long i;
  for (i=0; i<a.n; i++)
    a[i] = labs(a[i]);
}



/**
   The function prints out the contents of the %vector u to the out file,
   Optionally, the file, precision and field width can be specified file.
   
   @param u is the structure of the %vector which is printed
   
   @b Optionally :
   @param out   is the structure with opened file for %vector output (default is stdout)
   @param prec  is desired precision of the %vector elements on the output (default is 3)
   @param width is desired width of the %vector elements on the output (default is 11)
   
   @retval 0 : on succes
   @retval 1 : in case NULL pointer of the out parameter
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long printv(const vector &u, FILE *out, int prec, int width)
{
  if (out == NULL)
  {
    print_err("cannot print vector - parameter for output is a NULL pointer", 
              __FILE__, __LINE__, __func__);
    return (1);
  }
  for (long i = 0; i < u.n; i++)
    fprintf(out, "%*.*e\n", width, prec, u.a[i]);
  return (0);
}



/**
   The function prints out the contents of the %vector u given by array to the out file.
   Optionally, file, precision and field width can be specified
   
   @param u - pointer to array of double values which will be printed
   @param n - size of array u
   
   @b Optionally :
   @param out   is the structure with opened file for %vector output (default is stdout)
   @param prec  is desired precision of the %vector elements on the output (default is 3)
   @param width is desired width of the %vector elements on the output (default is 11)
   
   @retval 0 : on succes
   @retval 1 : in case NULL pointer of the out parameter
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long printv(const double *u, long n, FILE *out, int prec, int width)
{
  if (out == NULL)
  {
    print_err("cannot print vector - parameter for output is a NULL pointer", 
              __FILE__, __LINE__, __func__);
    return (1);
  }
  for (long i = 0; i < n; i++)
    fprintf(out, "%*.*e\n", width, prec, u[i]);
  return (0);
}



/**
   The function prints contents of the %vector u into the file out,
   
   @param out -  pointer to the structure with opened output file
   @param u   -  %vector which contents will be printed
   
   @b Requirements :
   Vector u have to be allocated memory.
   Number of read values is given by the %vector n member.
   
   @retval 0 : on succes
   @retval 1 : in case NULL pointer of the in parameter
   @retval 2 : in case error reading number from the file
   
   TKr, 11/02/2013 accroding to readv(XFILE *in, vector u)
*/
long printv(FILE *out, vector &u)
{
  for (long i = 0; i < u.n; i++)
  {
    fprintf(out, "  %le", u[i]);
  }
  return (0);
}



/**
   The function prints out the contents of the %ivector u to the file out.
   Optionally, the file and field width can be specified.
   
   @param u is the structure of the %ivector which is printed
   
   @b Optionally :
   @param out   is the structure with opened file for %ivector output (default is stdout)
   @param width is desired width of the %ivector elements on the output (default is 11)
   
   @retval 0 on succes
   @retval 1 in case NULL pointer of the out parameter
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long printv(const ivector &u, FILE *out, int width)
{
  if (out == NULL)
  {
    print_err("cannot print ivector - parameter for output is a NULL pointer", 
              __FILE__, __LINE__, __func__);
    return (1);
  }
  for (long i = 0; i < u.n; i++)
    fprintf(out, "%*ld\n", width, u.a[i]);
  return (0);
}



/**
   The function prints out the contents of the %vector u (integer values are assumed) 
   given by array. Optionally, the file and field width can be specified.
   
   @param u - pointer to array of integer values which will be printed
   @param n - size of array u
   
   @b Optionally :
   @param out   is the structure with opened file for %ivector output (default is stdout)
   @param width is desired width of the %ivector elements on the output (default is 11)
   
   @retval 0 on succes
   @retval 1 in case NULL pointer of the out parameter
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long printv(const long *u, long n, FILE *out, int width)
{
  if (out == NULL)
  {
    print_err("cannot print vector - parameter for output is a NULL pointer", 
              __FILE__, __LINE__, __func__);
    return (1);
  }
  for (long i = 0; i < n; i++)
    fprintf(out, "%*ld\n", width, u[i]);
  return (0);
}



/**
   The function reads contents of the %vector u from the file in,
   
   @param in    pointer to the structure with opened input file
   @param u     %vector which contents will be read
   
   @b Requirements :
   Vector u have to be allocated memory.
   Number of read values is given by the %vector n member.
   
   @retval 0 : on succes
   @retval 1 : in case NULL pointer of the in parameter
   @retval 2 : in case error reading number from the file
   
   created  3.12.2006 by Jaroslav Kruis, jk@cml.fsv.cvut.cz
*/
long readv(XFILE *in, vector &u)
{
  if (in == NULL)
  {
    print_err("cannot read vector - parameter for input file is a NULL pointer", 
              __FILE__, __LINE__, __func__);
    return (1);
  }
  for (long i = 0; i < u.n; i++)
  {
    if (xfscanf(in, "%le", &u.a[i]) != 1)
      return(2);
  }
  return (0);
}


/**
   The function reads contents of the %vector u from the string in,

   @param in    pointer to the input string
   @param u     vector which contents will be read
   
   @b Requirements :
   Vector u have to be allocated memory.
   Number of read values is given by the vector n member.
   
   @retval 0 : on succes
   @retval 1 : in case NULL pointer of the in parameter
   @retval 2 : in case error reading number from the file
   
   created  6.9.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long readv(char *in, vector &u)
{
  if (in == NULL)
  {
    print_err("cannot read vector - parameter for input string is a NULL pointer", 
              __FILE__, __LINE__, __func__);
    return (1);
  }
  for (long i = 0; i < u.n; i++)
  {
    if (sscanf(in, "%le", &u.a[i]) != 1)
      return(2);
  }
  return (0);
}



/**
   function extracts ncomp components from %vector b
   and put them into %vector a

   @param a - %vector containing extracted components
   @param b - %vector from where components are extracted
   @param fi - first index
   @param ncomp - number of components

   30.11.2002
*/
void extract (vector &a,vector &b,long fi, long ncomp)
{
  long i,j;

  j=fi;
  for (i=0;i<ncomp;i++){
    a[i]=b[j];
    j++;
  }
}



/**
   Function extracts positive components from vector a
   and puts them into vector b in corresponding position as in the vector a.
   If component is negative the zero is stored.

   @param a - %vector containing compared components
   @param b - %vector where extracted components are stored

   3.3.2003
*/
long extractposv (vector &a,vector &b)
{
  long i;
  if (a.n != b.n)
  {
    print_err("cannot extract positive components - incompatible dimensions of vectors\n"
              "a(%ld) X b(%ld)", __FILE__, __LINE__, __func__, a.n, b.n);
    return (1);
  }
  for (i = 0; i < a.n; i++)
  {
    if (a[i] > 0.0)
      b[i] = a[i];
    else
      b[i] = 0.0;
  }
  return 0;
}



/**
   Function extracts negative components from vector a
   and puts them into vector b in corresponding position as in the vector a.
   If component is positive the zero is stored.

   @param a - %vector containing compared components
   @param b - %vector where extracted components are stored

   25.2.2004
*/
long extractnegv (vector &a,vector &b)
{
  long i;
  if (a.n != b.n)
  {
    print_err("cannot extract negative components - incompatible dimensions of vectors\n"
              "a(%ld) X b(%ld)", __FILE__, __LINE__, __func__, a.n, b.n);
    return (1);
  }
  for (i = 0; i < a.n; i++)
  {
    if (a[i] < 0.0)
      b[i] = a[i];
    else
      b[i] = 0.0;
  }
  return 0;
}



/**
  The function sorts ivector v by Shell sort algorithm in ascending order.

  @param v - vector of sorted numbers

  @return The function does not return anything but changes order of v elements.

  Created by Tomas Koudelka, 2.7.2013
*/
void shell_sort(ivector &v)
{
  long flag = 1, d = v.n, i, temp;

  while(flag || (d>1))   // boolean flag (true when not equal to 0)
  {
    flag = 0;           // reset flag to 0 to check for future swaps
    d = (d+1)/2;
    for (i=0; i<(v.n-d); i++)
    {
      if (v[i+d] < v[i])
      {
        temp   = v[i+d]; // swap items at positions i+d and d
        v[i+d] = v[i];
        v[i]   = temp;
        flag   = 1;        // indicate that a swap has occurred
      }
    }
  }
  return;
}



/**
  The function sorts array of integer numbers v by Shell sort algorithm in ascending order.

  @param v - array of sorted numbers
  @param n - length of array v

  @return The function does not return anything but changes order of v elements.

  Created by Tomas Koudelka, 3.7.2013
*/
void shell_sort(long *v, long n)
{
  long flag = 1, d = n, i, temp;

  while(flag || (d>1))   // boolean flag (true when not equal to 0)
  {
    flag = 0;           // reset flag to 0 to check for future swaps
    d = (d+1)/2;
    for (i=0; i<(n-d); i++)
    {
      if (v[i+d] < v[i])
      {
        temp   = v[i+d]; // swap items at positions i+d and d
        v[i+d] = v[i];
        v[i]   = temp;
        flag   = 1;        // indicate that a swap has occurred
      }
    }
  }
  return;
}



/**
   function reads number of attributes and order of attributes used in stochastic or fuzzy computations
   
   @param in - pointer to input file
   
   JK, 23.8.2005
*/
long atsel::read(XFILE *in)
{
  long i;
  
  //  number of all attributes
  //  number of basic attributes
  //  number of hidden attributes
  //
  //  number of basic attributes should be usually equal to the number of all attributes
  //  it differs in case of plastic material, where parameters of the model are stored first (nba components)
  //  and hardening/softening parameters are stored behind them (nia components)
  xfscanf(in, "%ld %ld %ld", &num,&nba,&nia);
  
  if (nba+nia != num)
    print_err("wrong number of all attributes, basic attributes and hidden attributes",
              __FILE__, __LINE__, __func__);
  
  atrib = new long[num];
  for (i = 0; i < num; i++){
    xfscanf(in, "%ld", atrib+i);
    atrib[i]--;
  }
  return 0;
}
