#ifndef DEF_VECTOR
#define DEF_VECTOR
#include <stdio.h>
#include <assert.h>
#include <alloca.h>
#define DEBUG_VECTOR

#include <string.h>
#include "iotools.h"

#ifdef DEBUG_VECTOR
 extern unsigned long Acv; ///< alocation counter for vector class
 extern unsigned long Avmax; ///< peak of alocation memory of all vector instances
 extern unsigned long Ava; ///< actual allocated memory of all vector instances

 extern unsigned long Aciv; ///< alocation counter for ivector class
 extern unsigned long Aivmax; ///< peak of alocation memory of all ivector instances
 extern unsigned long Aiva; ///< actual allocated memory of all ivector instances
#endif

struct vector
/**
   This file declares struct of the vector, which implements
   %vector with %elemnts type of double.
   There is also declarations of the functions for the %vector computing.
*/
{
  long n;    ///< the number of %vector entries (components)
  double *a; ///< pointer to memory ,where are stored elements of %vector
  long size; ///< real length of array a (due to memory reallocation)
  
  vector() { n = size = 0L; a = NULL;}; ///< default constructor

  /**
     The constructor allocates memory form the heap to the member array a.
   
     @param n is the number of %vector elements
   
     created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
     modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  vector(long n) __attribute__((always_inline))
  {
    vector::n = n;
    a = (double *)alloca(sizeof(*a)*n);
    if (a == NULL)
      print_err("cannot allocate memory for vector v(%ld)", __FILE__, __LINE__, __func__, n);
    size = n;
    memset (a,0,n*sizeof(double));

    #ifdef DEBUG_VECTOR
     Acv++;
     Ava += n;
     if (Ava > Avmax)
       Avmax = Ava;
    #endif
  };

  /**
    The copy constructor creates copy of the object given by the constant reference parameter.
   
    @param v is reference to the object of the vector which should be copied
   
    created  16.10.2003 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  vector(const vector &v) __attribute__((always_inline))      ///< copy constructor
  {
    long i;
    if (v.n > size)
    {
      n = v.n;
      a = (double *)alloca(sizeof(*a)*n);
      if (a == NULL)
        print_err("cannot allocate memory for vector v(%ld)", __FILE__, __LINE__, __func__, n);
      size = v.n;
      #ifdef DEBUG_VECTOR
        print_warning("copy constructor of vector is called.\n"
                      "Please check your code and make sure you want use copy constructor.", __FILE__, __LINE__, __func__);
        Acv++;
        Ava += n;
        if (Ava > Avmax)
          Avmax = Ava;
      #endif
    }
    for (i = 0; i < n; i++)
      a[i] = v.a[i];
  };
  
  /**
     The function operator enables access to the elements of the member array a
     
     @param i is the number of the desired %vector element
     @retval Returns i-th element of %vector
     
     created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
     modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  double& operator () (long i)
  {
    #ifdef DEBUG_VECTOR
    if ((i >= n) || (i < 0))
       print_err("vector row index %ld is out of range <0,%ld>", __FILE__, __LINE__, __func__, i, n-1);

     assert(i < n);
    #endif
    return (a[i]);
  };
  
  /**
     The array index operator enables access to the elements of the member array a
     
     @param i is the number of the desired %vector %element
     @retval i-th %element of %vector
     
     created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
     modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  double& operator [] (long i)
  {
    #ifdef DEBUG_VECTOR
     if  ((i >= n) || (i < 0))
       print_err("vector row index %ld is out of range <0,%ld>", __FILE__, __LINE__, __func__, i, n-1);

     assert(i < n);
    #endif
    return (a[i]);
  };
  
  /**
     The array index operator enables CONSTANT access to the elements of the member array a
     
     @param i is the number of the desired %vector element
     @retval i-th %element of %vector
     
     created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
     modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  double operator [] (long i) const
  {
    #ifdef DEBUG_VECTOR
     if  ((i >= n) || (i < 0))
       print_err("vector row index %ld is out of range <0,%ld>", __FILE__, __LINE__, __func__, i, n-1);

     assert(i < n);
    #endif
    return (a[i]);
  };

  double get_item(long i) { return a[i]; }
  void set_item(long i, double val) { a[i] = val; }
  
  ~vector(); ///< destructor
};


struct ivector
/**
   This file declares struct of the vector, which implements
   %vector with %elemnts type of long.
*/
{
  long n;   ///< number of %vector elements
  long *a;  ///< pointer to memory ,where are stored elements of %vector
  long size; ///< real length of array a (due to memory reallocation)
  
  ivector() { n = size = 0L; a = NULL;}; ///< default constructor
  ivector(long n);               ///< allocating constructor
  ivector(const ivector &v);     ///< copy constructor
  
  /**
     The function operator enables access to the elements of the member array a
     
     @param i is the number of the desired %vector element
     @retval Returns i-th element of %vector
     
     created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
     modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  long& operator () (long i)
  {
    #ifdef DEBUG_VECTOR
     if  ((i >= n) || (i < 0))
       print_err("ivector row index %ld is out of range <0,%ld>", __FILE__, __LINE__, __func__, i, n-1);

     assert(i < n);
    #endif
    return (a[i]);
  };

  /**
   The array index operator enables access to the elements of the member array a
   
   @param i is the number of the desired %vector %element
   @retval i-th %element of %vector
   
   created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  long& operator [] (long i)
  {
    #ifdef DEBUG_VECTOR
     if  ((i >= n) || (i < 0))
       print_err("ivector row index %ld is out of range <0,%ld>", __FILE__, __LINE__, __func__, i, n-1);

     assert(i < n);
    #endif
    return (a[i]);
  };

  /**
   The array index operator enables CONSTANT access to the elements of the member array a
   
   @param i is the number of the desired %vector %element
   @retval i-th %element of %vector
   
   created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  long& operator [] (long i) const
  {
    #ifdef DEBUG_VECTOR
     if  ((i >= n) || (i < 0))
       print_err("ivector row index %ld is out of range <0,%ld>", __FILE__, __LINE__, __func__, i, n-1);

     assert(i < n);
    #endif
    return (a[i]);
  };

  ~ivector(); ///< destructor

  long get_item(long i) { return a[i]; }
  void set_item(long i, long val) { a[i] = val; }
};

/**
   this structure serves for defined attributes
   it is used e.g. in stochastic analysis
   
   TKo
*/
struct atsel{
  ///  number of components
  long num;
  ///  number of basic components (optional parameter)
  long nba;
  ///  number of internal (hidden) components (optional parameter)
  long nia;

  long *atrib;

  atsel (){
    num=0;  
    nba=0;  
    nia=0;  
    atrib=NULL;
  };
  ~atsel (){  delete [] atrib;  };

  long read(XFILE* in);
};

#ifdef DEBUG_VECTOR
 long give_acv();
long give_avmax();
#endif


/**
   The function allocates memory from heap to the %vector's member array a.
   
   @param n   is the number of the vector elements
   @param vec is the structure for allocated vector
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
inline long allocv(long n, vector &vec) __attribute__((always_inline));
inline long allocv(long n, vector &vec)
{
  vec.n = n;
  vec.a = (double *)alloca(sizeof(*vec.a)*n);
  if (vec.a == NULL)
  {
    print_err("cannot allocate memory for vector v(%ld)", __FILE__, __LINE__, __func__, n);
    return (1);
  }
  vec.size = n;
  memset (vec.a,0,n*sizeof(double));
  #ifdef DEBUG_VECTOR
   Acv++;
   Ava += n;
   if (Ava > Avmax)
     Avmax = Ava;
  #endif
  return (0);
};



/**
   The function reallocates memory for the vector's member array a.
   If the new vector size is greater than previously allocated one, 
   array a is deleted and new memory is allocted from the heap. 
   If the new vector size is less or equal, then the memory 
   allocated for the array a is left and the number of vector components 
   is changed. Components of the array a are set to zero.
   
   
   @param n   is the new number of the ivector components
   @param vec is the structure for reallocated vector
   
   @retval 0 - on success
   @retval 1 - if fails allocating memory
   
   created  12.2012 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
inline long reallocv(long n, vector &vec) __attribute__((always_inline));
inline long reallocv(long n, vector &vec)
{
  if (n > vec.size)
  {
    vec.a = (double *)alloca(sizeof(*vec.a)*n);
    if (vec.a == NULL)
    {
      print_err("cannot reallocate memory for vector v(%ld)", __FILE__, __LINE__, __func__, n);
      return (1);
    }
    vec.size = n;
    #ifdef DEBUG_VECTOR
     Acv++;
     Ava += n;
     if (Ava > Avmax)
       Avmax = Ava;
    #endif
  }
  vec.n = n;
  memset (vec.a,0,n*sizeof(*vec.a));
  return (0);
};


/// allocates %ivector
long allocv(long n, ivector &vec);
/// reallocates %ivector
long reallocv(long n, ivector &vec);

/// copies contents of %vector
long copyv(const vector &src, vector &dest);
/// copies contents of %ivector
long copyv(const ivector &src, ivector &dest);
long copyv(const long *src,long *dest,long n);
/// copies contents of array
long copyv(const double *src, double *dest, long n);
/// copies contents of array to vector
long copyv(const double *src, vector &dest);
/// copies contents of vector to array
long copyv(const vector &src, double *dest);

/// copies contents of %vector multiplied by scalar
long copymultv(const vector &src, vector &dest, const double c);
/// copies contents of %ivector multiplied by scalar
long copymultv(const ivector &src, ivector &dest, const long c);
/// copies contents of array multiplied by scalar
long copymultv(const double *src, double *dest, const double c, long n);

/// fills contents of %vector with value c
long fillv(double c, vector &vec);
/// fills contents of %ivector with value c
long fillv(long c, ivector &vec);
/// fills contents of integer %vector given by array with value c
void fillv(long c, long *vec,long n);
/// fills contents of %vector given by array with value c
void fillv(double c, double *vec,long n);

/// fills array with zeros
long nullv (double *a,long n);
long nullv (ivector &a);

/// deallocates %vector
long destrv(vector &vec);
/// deallocates %ivector
long destrv(ivector &vec);

/// adds 2 vectors c = a+b
long addv(const vector &a, const vector &b, vector &c);
/// adds 2 double arrays, 2nd array is multplied by constant a = a + bc*b
void addmultv(const vector &a, const vector &b, double bc, vector &c);
/// adds 2 double arrays, 2nd array is multplied by constant a = a + bc*b
void addmultv(vector &a, const vector &b, double bc);

/// adds 2 ivectors c = a+b
long addv(const ivector &a, const ivector &b, ivector &c);

/// adds 2 double arrays a = a+b
void addv(double *a,double *b,long n);

/// adds 2 double arrays c = a+b
void addv(double *a,double *b, double *c, long n);

/// adds 2 double arrays, 2nd array is multplied by constant and added to the first array
void addmultv (double *a,double *b,double bc,long n);
/// adds 2 double arrays, 2nd array is multplied by constant, the result is stored in the third array
void addmultv (double *a,double *b,double bc, double *c, long n);

/// subtracts 2 vectors
long subv(const vector &a, const vector &b, vector &c);
/// subtracts 2 vectors
long subv(const vector &a, const vector &b);
/// subtracts 2 ivectors
long subv(const ivector &a, const ivector &b, ivector &c);

/// subtracts 2 arrays
void subv(double *a,double *b,long n);
/// subtracts 2 arrays, result is stored to the another array
void subv (double *a,double *b,double *c,long n);

/// performs cross product of 2 vectors
long crprd(const vector &a, const vector &b, vector &c);
/// performs cross product of 2 ivectors
long crprd(const ivector &a, const ivector &b, ivector &c);

/// performs scalar product of 2 vectors
long scprd(const vector &a, const vector &b, double &scalar);
/// performs scalar product of 2 ivectors
long scprd(const ivector &a, const ivector &b, long &scalar);
/// performs scalar product of 2 arrays
double scprd(const double *a, const double *b, long n);
/// performs scalar product of 2 arrays
double ss (double *a,double *b,long n);

/// multiplies %vector by constant
long cmulv(double c, const vector &a, vector &u);
/// multiplies %ivector by constant of type double
long cmulv(double c, const ivector &a, vector &u);
/// multiplies %ivector by constant of type long
long cmulv(long c, const ivector &a, ivector &u);

/// multiplies %vector by constant
long cmulv(double c, vector &a);
/// multiplies %ivector by constant of type long
long cmulv(long c, ivector &a);
/// multiplies double array by constant
void cmulv(double c, double *a, long n);
/// multiplies double array by constant
void cmulv(double c, double *a, double *u, long n);

/// computes norm of %vector
double normv(const vector &a);
/// computes norm of %vector
double normv(const double *a, long n);
/// computes norm of %ivector
double normv(const ivector &a);

/// cosine angle of 2 vectors
long cosav(const vector &a, const vector &b, double &cos);
/// cosine angle of 2 ivectors
long cosav(const ivector &a, const ivector &b, double &cos);

/// absolute value of vector components
void absv(vector &a);
/// absolute value of vector components
void absv(ivector &a);

/// prints contents of %vector to the file out with given precision and field width
long printv(const vector &u, FILE *out = stdout, int prec = 3, int width = 11);
/// prints contents of %vector given by array to the file out with given precision and field width
long printv(const double *u, long n, FILE *out = stdout, int prec = 3, int width = 11);
/// prints %vector u to the file out
long printv(FILE *out, vector &u);
/// prints contents of %ivector
long printv(const ivector &u, FILE *out = stdout, int width = 11);
/// prints contents of integer %vector given by array
long printv(const long *u, long n, FILE *out = stdout, int width = 11);

/// reads contents of %vector from file
long readv(XFILE *in, vector &u);
/// reads contents of %vector from string
long readv(char *in, vector &u);

/// function extracts ncomp components (from index fi) from %vector b
void extract (vector &a, vector &b, long fi, long ncomp);

/// extracts only positive components from %vector a and stores them to %vector b
long extractposv (vector &a, vector &b);
/// extracts only negative components from %vector a and stores them to %vector b
long extractnegv (vector &a, vector &b);

/// Shell sort of ivector
void shell_sort(ivector &v);

/// Shell sort for long integer array
void shell_sort(long *v, long n);

#endif
